package org.processmining.security.plugins.interDataProcessPolicy;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.deckfour.xes.model.XEvent;
import org.processmining.log.csv.CSVFile;
import org.processmining.partialorder.models.palignment.Move;
import org.processmining.security.plugins.Constants;
import org.processmining.security.plugins.Utility;

public class CRUD {
	private HashMap<String, List<CRUDEntry>> crudMap;
	private HashMap<String, List<CRUDEntry>> crudMapRequiredActions;
	private String dataHeader[];
	private List<String> activities;
	private List<String> operations;
	private List<String> dataObjects;
	public CRUD(CSVFile crudCSVFile) throws FileNotFoundException, IOException{
		parseCSVFile(crudCSVFile);
	}
	private void parseCSVFile(CSVFile crudCSVFile) throws FileNotFoundException, IOException{
		activities = new ArrayList<String>();
		operations = new ArrayList<String>();
		dataObjects = new ArrayList<String>();
		crudMap = new HashMap<String, List<CRUDEntry>>();
		crudMapRequiredActions = new HashMap<String, List<CRUDEntry>>();
		BufferedReader in = new BufferedReader(new InputStreamReader(crudCSVFile.getInputStream()));
		String line = in.readLine();
		if(line == null){
			//todo: Input validation
			return;
		}
		dataHeader = line.split(",");
		for(int i = 1; i < dataHeader.length; i++)
			dataObjects.add(dataHeader[i]);
		while((line = in.readLine()) != null) {
			String row[] = line.split(",");
			List<CRUDEntry> crudEntryList = new ArrayList<CRUDEntry>();
			List<CRUDEntry> crudEntryListRequiredActions = new ArrayList<CRUDEntry>();
			for(int i = 1; i < row.length; i++)
				for(int j = 0; j < row[i].length(); j++){
					if(!operations.contains(row[i].substring(j, j+1).toLowerCase()))
						operations.add(row[i].substring(j, j+1).toLowerCase());
					CRUDEntry crudEntry = new CRUDEntry(dataHeader[i], row[i].substring(j, j+1));
					crudEntryList.add(crudEntry);
					if(!crudEntry.isOptional())
						crudEntryListRequiredActions.add(crudEntry);
				}				
			crudMap.put(row[0], crudEntryList);
			crudMapRequiredActions.put(row[0], crudEntryListRequiredActions);
			activities.add(row[0]);
		}
	}
	public List<String> getOperations(){
		return operations;
	}
	public List<String> getActivities(){
		return activities;
	}
	public List<String> getDataObjects(){
		return dataObjects;
	}
	public List<CRUDEntry> getAllowedActions(String transition){
		return crudMap.get(transition);
	}
	public List<CRUDEntry> getAllowedActions(Move move){		
		return crudMap.get(Utility.getActivity(move));
	}	
	public List<CRUDEntry> getRequiredActions(String transition){
		return crudMapRequiredActions.get(transition);
	}
	public List<CRUDEntry> getRequiredActions(Move move){
		return crudMapRequiredActions.get(Utility.getActivity(move));
	}
	public CRUDEntry crudCheck(String transitionName, XEvent systemEvent){
		if(transitionName == null)
			return null;
		List<CRUDEntry> crudEntryVector = getAllowedActions(transitionName);	
		if(crudEntryVector == null)
			return null;
		for (CRUDEntry crudEntry : crudEntryVector) {
			if(crudEntry.isPermitted(Utility.getAttributeValue(systemEvent, Constants.DATA), 
					Utility.getAttributeValue(systemEvent, Constants.OPERATION)))
				return crudEntry;
		}
		return null;
	}
	public HashMap<String, List<CRUDEntry>> getCrudMap() {
		return crudMap;
	}
}
