package org.processmining.security.plugins.ui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.processmining.framework.util.ui.widgets.ProMComboBox;
import org.processmining.security.plugins.InterLevelConformanceCheckingPlugin;
import org.processmining.security.plugins.criterion.Criteria;
import org.processmining.security.plugins.criterion.CriteriaInterface;
import org.processmining.security.plugins.criterion.CriterionFunction;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class CriterionConfigurationPanel extends JPanel {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private final JLabel label;
	private ProMComboBox<String> attributeComboBox1[], attributeComboBox2[], operationComboBox[];
	private ProMComboBox <AttributeType> type[];
	private JCheckBox[] checkBox;
	public CriterionConfigurationPanel(Iterable<String> pAttributes, Iterable<String> sAttributes) {
		label = SlickerFactory.instance().createLabel("Define criterion that have to be met:");				
		setLayout(new GridBagLayout());
		setOpaque(false);
		GridBagConstraints c = new GridBagConstraints();		
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 3;
		c.fill = GridBagConstraints.VERTICAL;
		add(label, c);
		
		checkBox = new JCheckBox[3];
		attributeComboBox1 = new ProMComboBox[3];
		operationComboBox = new ProMComboBox[3];
		attributeComboBox2 = new ProMComboBox[3];
		type = new ProMComboBox[3];
				
		for(int i = 0; i < 3; i++){
			attributeComboBox1[i] = new ProMComboBox<String>(pAttributes);
			operationComboBox[i] = new ProMComboBox<String>(new String[]{"=", "<=", ">="});
			attributeComboBox2[i] = new ProMComboBox<String>(sAttributes);
			type[i] = new ProMComboBox<AttributeType>(AttributeType.values());
			checkBox[i] = SlickerFactory.instance().createCheckBox("", true); 
					
			attributeComboBox1[i].setPreferredSize(new Dimension(200, 30));
			operationComboBox[i].setPreferredSize(new Dimension(100, 30));
			attributeComboBox2[i].setPreferredSize(new Dimension(200, 30));
			type[i].setPreferredSize(new Dimension(100, 30));
			
			GridBagConstraints c0 = new GridBagConstraints();		
			c0.gridx = 0;
			c0.gridy = i+2;
			c0.fill = GridBagConstraints.VERTICAL;
			add(checkBox[i], c0);
			
			GridBagConstraints c1 = new GridBagConstraints();		
			c1.gridx = 1;
			c1.gridy = i+2;
			c1.gridwidth = 3;
			c1.fill = GridBagConstraints.VERTICAL;
			add(attributeComboBox1[i], c1);
			
			GridBagConstraints c2 = new GridBagConstraints();		
			c2.gridx = 4;
			c2.gridy = i+2;
			c2.gridwidth = 1;
			c2.fill = GridBagConstraints.VERTICAL;
			add(operationComboBox[i], c2);
			
			GridBagConstraints c3 = new GridBagConstraints();		
			c3.gridx = 5;
			c3.gridy = i+2;
			c3.gridwidth = 3;
			c3.fill = GridBagConstraints.VERTICAL;
			add(attributeComboBox2[i], c3);
			
			GridBagConstraints c4 = new GridBagConstraints();		
			c4.gridx = 8;
			c4.gridy = i+2;
			c4.gridwidth = 1;
			c4.fill = GridBagConstraints.VERTICAL;
			add(type[i], c4);
		}
		if(!InterLevelConformanceCheckingPlugin.FID_AVAILABLE)
			checkBox[0].setSelected(false);
		String a1[] = {"concept:name", "start", "complete"};
		String a2[] = {"fid", "time:timestamp", "time:timestamp"};
		String op[] = {"=", "<=", ">="};
		AttributeType ty[] = {AttributeType.String, AttributeType.Date, AttributeType.Date};
		for(int i = 0; i < 3; i++){
			attributeComboBox1[i].setSelectedItem(a1[i]);
			operationComboBox[i].setSelectedItem(op[i]);
			attributeComboBox2[i].setSelectedItem(a2[i]);
			type[i].setSelectedItem(ty[i]);
		}
	}
	public enum AttributeType {
		Date, Double, String
	}
	public CriterionFunction getCriterion(){
		Vector<CriteriaInterface> criterion = new Vector<CriteriaInterface>();
		for(int i = 0; i < 3; i++){
			if(checkBox[i].isSelected()){
				CriteriaInterface criteria = new Criteria(attributeComboBox1[i].getSelectedItem().toString(), 
					attributeComboBox2[i].getSelectedItem().toString(), operationComboBox[i].getSelectedItem().toString(), (AttributeType) type[i].getSelectedItem());
				criterion.add(criteria);
			}
		}
		return new CriterionFunction(criterion);
	}
}
